"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$HTML = exports.$capitalize = exports.$firstcap = exports.$normspaces = exports.$lines = exports.$trim = exports.$ftrim = exports.$rtrim = exports.$ltrim = exports.$ascii = void 0;
const commons_1 = require("./commons");
const string_tables_1 = require("./string_tables");
// for now $ascii() does not mak any transliterations from
// non-latin languages like Greek
function $ascii(source) {
    const l = (0, commons_1.$length)(source);
    if (!l)
        return '';
    let s = source.replace(/≠/g, "");
    s = s.normalize("NFD").replace(/[\u0300-\u036f]|\u00a8|\u00b4/g, "").normalize("NFKD"); // does most of the job
    // finally we will try to convert (or remove) the remaining non ascii characters
    return s.replace(/[^\x00-\x7F]/g, x => string_tables_1.FoundationASCIIConversion[x] || '');
}
exports.$ascii = $ascii;
/**
 *   We don't use standard trim because it does not trim all unicode whitespaces!
 */
// left-trim
function $ltrim(s) { return (0, commons_1.$length)(s) ? s.replace(string_tables_1.FoundationLeftTrimRegex, "") : ''; }
exports.$ltrim = $ltrim;
// right-trim
function $rtrim(s) { return (0, commons_1.$length)(s) ? s.replace(string_tables_1.FoundationRightTrimRegex, "") : ''; }
exports.$rtrim = $rtrim;
// full-trim
function $ftrim(s) { return (0, commons_1.$length)(s) ? s.replace(string_tables_1.FoundationLeftTrimRegex, "").replace(string_tables_1.FoundationRightTrimRegex, "") : ''; }
exports.$ftrim = $ftrim;
exports.$trim = $ftrim;
function $lines(s) { return (0, commons_1.$ok)(s) ? s.split(string_tables_1.FoundationNewLinesSplitRegex) : []; }
exports.$lines = $lines;
function $normspaces(s) { return $ftrim(s).replace(string_tables_1.FoundationFindAllWhitespacesRegex, " "); }
exports.$normspaces = $normspaces;
function $firstcap(s) { return _capitalize(s, 1); }
exports.$firstcap = $firstcap;
function $capitalize(s) { return _capitalize(s); }
exports.$capitalize = $capitalize;
function $HTML(s, reference = string_tables_1.FoundationHTMLEncoding) {
    const len = (0, commons_1.$length)(s);
    let ret = "";
    for (let i = 0; i < len; i++) {
        const c = s.charCodeAt(i);
        if (c < 0xff) {
            ret += reference[c];
        }
        else {
            ret += '&#' + c;
        }
    }
    return ret;
}
exports.$HTML = $HTML;
if (!('isWhiteSpace' in String.prototype)) {
    String.prototype.isWhiteSpace = function isNewLine() { return string_tables_1.FoundationWhiteSpacesStringCodeSet.has(this); };
    String.prototype.isStrictWhiteSpace = function isNewLine() { return string_tables_1.FoundationStrictWhiteSpacesStringCodeSet.has(this); };
}
if (!('isNewLine' in String.prototype)) {
    String.prototype.isNewLine = function isNewLine() { return this === '\u000a' || this === '\u000b' || this === '\u000c' || this === '\u000d'; };
}
if (!('ascii' in String.prototype)) {
    String.prototype.ascii = function ascii() { return $ascii(this); };
}
if (!('toHTML' in String.prototype)) {
    String.prototype.toHTML = function toHTML(reference) { return $HTML(this, reference); };
}
if (!('firstCap' in String.prototype)) {
    String.prototype.firstCap = function firstCap() { return $firstcap(this); };
}
if (!('capitalize' in String.prototype)) {
    String.prototype.capitalize = function capitalize() { return $capitalize(this); };
}
if (!('normalizeSpaces' in String.prototype)) {
    String.prototype.normalizeSpaces = function normalizeSpaces() { return $normspaces(this); };
}
if (!('ftrim' in String.prototype)) {
    String.prototype.ftrim = function ftrim() { return $ftrim(this); };
}
if (!('ltrim' in String.prototype)) {
    String.prototype.ltrim = function ltrim() { return $ltrim(this); };
}
if (!('rtrim' in String.prototype)) {
    String.prototype.rtrim = function rtrim() { return $rtrim(this); };
}
if (!('isDate' in String.prototype)) {
    String.prototype.isDate = function isDate() { return (0, commons_1.$ok)((0, commons_1.$isodate)(this)); };
}
if (!('isEmail' in String.prototype)) {
    String.prototype.isEmail = function isEmail() { return (0, commons_1.$ok)((0, commons_1.$email)(this)); };
}
if (!('isUrl' in String.prototype)) {
    String.prototype.isUrl = function isUrl() { return (0, commons_1.$ok)((0, commons_1.$url)(this)); };
}
if (!('isUUID' in String.prototype)) {
    String.prototype.isUUID = function isUUID() { return (0, commons_1.$ok)((0, commons_1.$UUID)(this)); };
}
if (!('lines' in String.prototype)) {
    String.prototype.lines = function lines() { return $lines(this); };
}
if (!('toInt' in String.prototype)) {
    String.prototype.toInt = function toInt(defaultValue) { return (0, commons_1.$toint)(this, defaultValue); };
}
if (!('toUnsigned' in String.prototype)) {
    String.prototype.toUnsigned = function toUnsigned(defaultValue) { return (0, commons_1.$tounsigned)(this, defaultValue); };
}
// ================================== private functions ==============================
function _capitalize(s, max = 0) {
    const len = (0, commons_1.$length)(s);
    let ret = "";
    if (!max) {
        max = len;
    }
    ;
    let lastCharWasNotLetter = true;
    let n = 0;
    for (let i = 0; i < len; i++) {
        const c = s.charAt(i);
        const isLetter = _charAssimilableAsLetter(c);
        if (isLetter && lastCharWasNotLetter && n < max) {
            ret += c.toUpperCase();
            n++;
        }
        else {
            ret += c;
        }
        lastCharWasNotLetter = !isLetter;
    }
    return ret;
}
function _charAssimilableAsLetter(c) {
    c = $ascii(c);
    if (c.length) {
        const v = c.charCodeAt(0) & ~32;
        return v >= 65 && v <= 90;
    }
    return false;
}
//# sourceMappingURL=strings.js.map